local L = LFM_SnifferLocals

LFM_Sniffer = CreateFrame("Frame")
LFM_SnifferDB = {enabled = true}

local frame = LFM_Sniffer
local map = {}
local selectedLFMType
local selectedLFMName

hooksecurefunc("LFMTypeButton_OnClick", function()
    table.wipe(map)
end)

local updateLFGMap = function()
    table.wipe(map)
    -- Set the upvalues
    selectedLFMType = UIDropDownMenu_GetSelectedID(LFMFrameTypeDropDown)
    selectedLFMName = UIDropDownMenu_GetSelectedID(LFMFrameNameDropDown)
    local offset = FauxScrollFrame_GetOffset(LFMListScrollFrame)
    local numResults, totalCount = GetNumLFGResults(selectedLFMType, selectedLFMName)
    for i = 1, numResults do
        if LFMFrameFilter_DataMatches(GetLFGResults(selectedLFMType, selectedLFMName, i)) then
            local realIndex = i + offset
            map[#map + 1] = realIndex
        end
    end
end

hooksecurefunc("LFMFrame_Update", updateLFGMap)

frame:RegisterEvent("UPDATE_LFG_LIST")
frame:RegisterEvent("UPDATE_LFG_LIST_INCREMENTAL")
frame:SetScript("OnEvent", function(self, event)
    updateLFGMap()
end)

local lines = setmetatable({}, {__index = function(tbl, key)
    local new = {}
    rawset(tbl, key, new)
    return new
end})

function AddSpacerLine(self, height, r, g, b, a)
        self:AddDoubleLine(" ", " ")
        local num = self:NumLines()
        local line = lines[self][num]
        if not line then
                line = self:CreateTexture(nil, "ARTWORK")
                line:SetPoint("LEFT" , self:GetName() .. "TextLeft" .. num, "LEFT")
                line:SetPoint("RIGHT", self:GetName() .. "TextRight" ..num, "RIGHT")
                lines[self][num] = line
        end
        line:SetHeight(height or 1)
        line:SetTexture(r or NORMAL_FONT_COLOR.r, g or NORMAL_FONT_COLOR.g, b or NORMAL_FONT_COLOR.b, a or 1)
        line:Show()
end

local function OnTooltipCleared(self)
    for k, line in pairs(lines[self]) do line:Hide() end
end

GameTooltip:HookScript("OnTooltipCleared", OnTooltipCleared)
GameTooltip:HookScript("OnTooltipCleared", OnTooltipCleared)

local infoLabels = {
    [22] = L["Armor: "],
    [23] = L["+Damage: "],
    [24] = L["+Healing: "],
    [25] = L["Melee crit: "],
    [26] = L["Ranged crit: "],
    [27] = L["Spell crit: "],
    [28] = L["Mana Regen: "],
    [29] = L["MP5 (combat): "],
    [30] = L["Attack power: "],
    [31] = L["Agility: "],
    [32] = L["Health: "],
    [33] = L["Power: "],
    [34] = L["Avg. Gear rating: "],
    [35] = L["Defense: "],
    [36] = L["Dodge: "],
    [37] = L["Block: "],
    [38] = L["Parry: "],
    [39] = L["Haste: "],
    [40] = L["Expertise: "],
}

local classInfo = {
    ["DEATHKNIGHT"] = { 22,23,25,30,31,32,34,35,36,38,39,40 },
    ["DRUID"] = { 22,23,24,25,27,28,29,30,31,32,33,34,36,39,40 },
    ["HUNTER"] = { 26,30,31,32,34,39 },
    ["MAGE"] = { 23,27,28,29,32,33,34,39 },
    ["PALADIN"] = { 22,23,24,25,27,28,29,30,31,32,33,34,35,36,37,38,39,40 },
    ["PRIEST"] = { 23,27,28,29,32,33,34,39 },
    ["ROGUE"] = { 23,25,30,31,32,34,36,38,39,40 },
    ["SHAMAN"] = { 23,24,25,27,28,29,30,31,32,33,34,36,39,40},
    ["WARLOCK"] = { 23,27,28,29,32,33,34,39 },
    ["WARRIOR"] = { 22,23,25,30,31,32,34,35,36,37,38,39,40 },
}

local mt = {__index = function(t, k)
    return "nil"
end}

local h = "|cff00d1ff%s|r"
local function decorateTooltip(tooltip, info, perm)
    local classFileName = info[11]

    if perm then
        local msg = string.format(L["%s - Level %d %s"], info[1], info[2], info[4])
        tooltip:SetText(msg)
    end
    
    AddSpacerLine(tooltip, 1)

    tooltip:AddLine(L["Sniffer Player Details:"])

    local talents = info[17] .. "/" .. info[18] .. "/" .. info[19]
    local bossKills = info[16]

    tooltip:AddLine(h:format("  " .. L["Talents: "]) .. talents, 1, 1, 0)
    tooltip:AddLine(h:format("  " .. L["Boss kills: "]) .. bossKills, 1, 1, 0)

    for k, v in pairs(classInfo[classFileName]) do
        tooltip:AddLine(h:format("  " .. infoLabels[v]) .. tostring(info[v]), 1, 1, 0)
    end

    tooltip:Show()
end

local info;
hooksecurefunc("LFMButton_OnEnter", function(self)
    local index = map[self:GetID()]

    if not index then
        return
    elseif not LFM_SnifferDB.enabled then
        return
    end

    local tooltip = GameTooltip
    local info = setmetatable({GetLFGResults(selectedLFMType, selectedLFMName, index)}, mt) 
    decorateTooltip(tooltip, info)
end)

hooksecurefunc("LFMButton_OnClick", function(self)
    if IsShiftKeyDown() then
        local index = map[self:GetID()]
        local info = setmetatable({GetLFGResults(selectedLFMType, selectedLFMName, index)}, mt) 

        local tooltip = ItemRefTooltip
        ShowUIPanel(tooltip)
        if not tooltip:IsShown() then 
            tooltip:SetOwner(UIParent, "ANCHOR_PRESERVE")
        end
        tooltip:ClearLines()
        decorateTooltip(tooltip, info, true)
    end
end)

SLASH_LFM_SNIFFER1 = "/lfmsniffer"
SlashCmdList["LFM_SNIFFER"] = function(msg, editbox)
    LFM_SnifferDB.enabled = not LFM_SnifferDB.enabled
    print("|cff00d1ff" .. L["LFM_Sniffer has been"] .. "|r |cffffff00" .. (LFM_SnifferDB.enabled and L["enabled"] or L["disabled"]) .. "|r")
end
